﻿using Microsoft.EntityFrameworkCore;

using TestLibrary.Models;

namespace TestServer.Database
{
    /// <summary>
    /// 데이터베이스 컨텍스트
    /// </summary>
    public class DatabaseContext : DbContext
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 사용자 테이블 - User

        /// <summary>
        /// 사용자 테이블
        /// </summary>
        public DbSet<UserModel> User { get; set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - DatabaseContext(options)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="options">DB 컨텍스트 옵션</param>
        public DatabaseContext(DbContextOptions<DatabaseContext> options) : base(options)
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 모델 생성시 처리하기 - OnModelCreating(builder)

        /// <summary>
        /// 모델 생성시 처리하기
        /// </summary>
        /// <param name="builder">모델 빌더</param>
        protected override void OnModelCreating(ModelBuilder builder)
        {
            base.OnModelCreating(builder);
            
            builder.Entity<UserModel>().HasData
            (
                new UserModel { ID = 1, UserName = "admin" , Password = "1234" },
                new UserModel { ID = 2, UserName = "tester", Password = "1234" }
            );
        }

        #endregion
    }
}