using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.IdentityModel.Tokens;
using System;
using System.Text;

using TestServer.Database;
using TestServer.Services;

namespace TestServer
{
    /// <summary>
    /// 
    /// </summary>
    public class Startup
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - Configuration

        /// <summary>
        /// 
        /// </summary>
        public IConfiguration Configuration { get; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - Startup(configuration)

        /// <summary>
        /// 
        /// </summary>
        /// <param name="configuration"></param>
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  ÷ ϱ - ConfigureServices(services)

        /// <summary>
        ///  ÷ ϱ
        /// </summary>
        /// <param name="services"> ÷</param>
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddDbContext<DatabaseContext>
            (
                options =>
                {
                    options.UseSqlServer(Configuration.GetConnectionString("DefaultConnection"));
                }
            );

            services.AddAuthentication
            (
                options =>
                {
                    options.DefaultAuthenticateScheme = JwtBearerDefaults.AuthenticationScheme;
                    options.DefaultChallengeScheme    = JwtBearerDefaults.AuthenticationScheme;
                }
            )
            .AddJwtBearer
            (
                options =>
                {
                    options.RequireHttpsMetadata      = true;
                    options.SaveToken                 = true;
                    options.TokenValidationParameters = new TokenValidationParameters()
                    {
                        IssuerSigningKey         = new SymmetricSecurityKey
                        (
                            Encoding.UTF8.GetBytes("placeholder-key-that-is-long-enough-for-sha256")
                        ),
                        ValidateAudience         = false,
                        ValidateIssuer           = false,
                        ValidateLifetime         = false,
                        RequireExpirationTime    = false,
                        ClockSkew                = TimeSpan.Zero,
                        ValidateIssuerSigningKey = true
                    };
                }
            );
            
            services.AddScoped<ITokenBuilder, TokenBuilder>();

            services.AddControllers();
        }

        #endregion
        #region ϱ - Configure(app, environment)

        /// <summary>
        /// ϱ
        /// </summary>
        /// <param name="app">ø̼ </param>
        /// <param name="environment"> νƮ ȯ</param>
        public void Configure(IApplicationBuilder app, IWebHostEnvironment environment)
        {
            if(environment.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseHttpsRedirection();

            app.UseRouting();

            app.UseAuthorization();

            app.UseEndpoints
            (
                endpoints =>
                {
                    endpoints.MapControllers();
                }
            );
        }

        #endregion
    }
}